<?php

namespace Drupal\gbif2_entity\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\external_entities\ExternalEntityInterface;
use Drupal\external_entities\ResponseDecoder\ResponseDecoderFactoryInterface;
use Drupal\external_entities\StorageClient\ExternalEntityStorageClientBase;
use ResTelae\Gbif\Occurrences;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * External entities storage client based on the GBIF REST API.
 *
 * @ExternalEntityStorageClient(
 *   id = "gbif",
 *   label = @Translation("GBIF"),
 *   description = @Translation("Retrieves GBIF records")
 * )
 */
class Gbif extends ExternalEntityStorageClientBase {

  /**
   * GBIF occurrences service, provided by the PHP GBIF library.
   *
   * @var \ResTelae\Gbif\Occurrences
   */
  protected $occ;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TranslationInterface $string_translation, ResponseDecoderFactoryInterface $response_decoder_factory, Occurrences $occ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $string_translation, $response_decoder_factory);
    $this->occ = $occ;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('external_entities.response_decoder_factory'),
      $container->get('gbif.occurrence')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function load($id) {
    return $this->occ->get($id);
  }

  /**
   * {@inheritdoc}
   */
  public function loadMultiple(array $ids = NULL) {
    $data = [];

    if (!empty($ids) && is_array($ids)) {
      foreach ($ids as $id) {
        $data[$id] = $this->load($id);
      }
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   *
   * @note GBIF API does not allow sorting.
   */
  public function query(array $parameters = [], array $sorts = [], $start = NULL, $length = NULL) {
    $results = $this->search($parameters, $start, $length);

    return $results['results'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function countQuery(array $parameters = []) {
    $results = $this->search($parameters, 0, 1);

    return $results['count'] ?? 0;
  }

  /**
   * Search from GBIF.
   *
   * @param array $parameters
   *   (optional) Array of parameters, each value is an array with the following
   *   key-value pairs:
   *     - field: the field name the parameter applies to
   *     - value: the value of the parameter
   *     - operator: the operator of how the parameter should be applied.
   * @param int|null $start
   *   (optional) The first item to return.
   * @param int|null $length
   *   (optional) The number of items to return.
   *
   * @return array
   *   Results retrieved from GBIF.
   */
  protected function search(array $parameters = [], $start = NULL, $length = NULL) {
    $params = [
      'limit' => $length,
      'offset' => $start,
    ];

    foreach ($parameters as $parameter) {
      // We do not bother with fancy operators.
      // @see \Drupal\gbif2_views\Plugin\views\filter\GbifNumericFilter
      $params[$parameter['field']] = $parameter['value'];
    }

    return $this->occ->search($params);
  }

  /**
   * {@inheritdoc}
   */
  public function save(ExternalEntityInterface $entity) {
    // Not currently supported.
  }

  /**
   * {@inheritdoc}
   */
  public function delete(ExternalEntityInterface $entity) {
    // Not currently supported.
  }

}
